<?php

namespace Drupal\ims;

use Drupal\Core\Database\Connection;
use GuzzleHttp\Exception\RequestException;

/**
 * Class ApiService.
 */
class IMSApiService {

  /**
   * Database connection.
   *
   * Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Constructs a new IMSApiService object.
   */
  public function __construct(Connection $connection) {
    $this->database = $connection;
  }

  /**
   *  Create an invoice.
   *
   *  @param boolean $p_general
   *  Invoice type
   *  @param string $id_sale
   *  Sale ID
   *
   */
  public function createInvoice($p_general, $id_sale) {
    $config =  \Drupal::config('ims.settings');
    $client = \Drupal::httpClient();
    $datosFactura = [];
    // Get payment details
    $ppss_sales = $this->database->select('ppss_sales_details', 'sd');
    $ppss_sales->join('ppss_sales', 's', 'sd.sid = s.id');
    $ppss_sales->condition('sd.id', $id_sale);
    $ppss_sales->fields('sd', ['id', 'created','tax', 'price', 'total']);
    $ppss_sales->fields('s', ['uid','details']);
    $sales = $ppss_sales->execute()->fetchAssoc();

    $details = json_decode($sales['details']);
    // Get user billing data
    $user = $this->database->select('ims_user_invoice', 'i')
            ->condition('uid', $sales['uid'])
            ->fields('i')
            ->execute()
            ->fetchAssoc();
    // Get the last folio
    $folio = $this->database->query("SELECT max(folio) as folio from {ims_invoices}")->fetchAssoc();
    // Get product data
    $product = \Drupal\node\Entity\Node::load($details->node_id);

    try {
      $conceptos = array();
      $unconcepto = array();
      $base = $sales['price'];
      $importe = $sales['total'];
      $impuesto = $sales['tax'];
      $unconcepto = [
        'ObjetoImp' => '02',
        'ClaveProdServ' => $product->get($config->get('field_prod_serv_code'))->getValue(),
        'NoIdentificacion' => '01',
        'Cantidad' => 1,
        'ClaveUnidad' => $product->get($config->get('field_unit_code'))->getValue(),
        'Descripcion' => $details->description,
        'ValorUnitario' => $base,
        'Importe' => $base,
        'Descuento' => 0
      ];
      $impuestosTraslados = array(
        'Base' => $base,
        'Impuesto' => '002',
        'TipoFactor' => 'Tasa',
        'TasaOCuota' => number_format($details->tax / 100, 6),
        'Importe' => $impuesto
      );
      $unconcepto['Impuestos']['Traslados'][0] = $impuestosTraslados;
      $conceptos[] = $unconcepto;

      $datosFactura['Version'] = '4.0';
      $datosFactura['Exportacion'] = '01';
      $datosFactura['Serie'] = $config->get('serie');
      $datosFactura['Folio'] = $folio['folio'] ? $folio['folio'] + 1 : $config->get('folio');
      $datosFactura['Fecha'] = 'AUTO';
      $datosFactura['FormaPago'] = $config->get('c_pago');
      $datosFactura['CondicionesDePago'] = "";
      $datosFactura['SubTotal'] = $base;
      $datosFactura['Descuento'] = null;
      $datosFactura['Moneda'] = strtoupper($details->currency);
      $datosFactura['TipoCambio'] = 1;
      $datosFactura['Total'] = $importe;
      $datosFactura['TipoDeComprobante'] = 'I';
      $datosFactura['MetodoPago'] = "PUE";
      $datosFactura['LugarExpedicion'] = $config->get('lugar_expedicion');
      # Optional data
      # Optional legend to put next to the folio: FACTURA, RECIBO, NOTA DE CREDITO, ETC.
      $datosFactura['LeyendaFolio'] = "FACTURA"; 
      # Regimen fiscal del emisor ligado al tipo de operaciones que representa este CFDI
      $datosFactura['Emisor']['RegimenFiscal'] = $config->get('regimen_fiscal');

      if($p_general) {
        # Required data
        $datosFactura['Receptor']['Rfc'] = 'XAXX010101000';
        $datosFactura['Receptor']['Nombre'] = 'PUBLICO EN GENERAL';
        $datosFactura['Receptor']['UsoCFDI'] = 'S01';
        // Must be the same as LugarExpedición
        $datosFactura["Receptor"]["DomicilioFiscalReceptor"] = $config->get('lugar_expedicion');
        $datosFactura["Receptor"]["RegimenFiscalReceptor"] = '616';
        // 01-Diaria, 02-Semanal, 03-Quincenal, 04-Mensual o 05-Bimestral
        $datosFactura["InformacionGlobal"]["Periodicidad"] = '04';
        $datosFactura["InformacionGlobal"]["Meses"] = date('m');
        $datosFactura["InformacionGlobal"]["Año"] = date('Y');

      } else {
        # Required customer data
        $datosFactura['Receptor']['Rfc'] = $user['rfc'];
        $datosFactura['Receptor']['Nombre'] = $user['name'];
        $datosFactura['Receptor']['UsoCFDI'] = $user['cfdi'];
        $datosFactura["Receptor"]["DomicilioFiscalReceptor"] = $user['postal_code'];
        $datosFactura["Receptor"]["RegimenFiscalReceptor"] = $user['regimen_fiscal'];

        # Optional customer data
        $datosFactura["Receptor"]["Calle"] = $user['address'];
        $datosFactura["Receptor"]["NoExt"] = $user['number_ext'];
        $datosFactura["Receptor"]["NoInt"] = $user['number_int'];
        $datosFactura["Receptor"]["Colonia"] = $user['neighborhood'];
        $datosFactura["Receptor"]["Loacalidad"] = $user['city'];
        $datosFactura["Receptor"]["Municipio"] = $user['municipality'];
        $datosFactura["Receptor"]["Estado"] = $user['state'];
        $datosFactura["Receptor"]["Pais"] = 'MEXICO';
        $datosFactura["Receptor"]["CodigoPostal"] = $user['postal_code'];
      }

      $datosFactura['Conceptos'] = $conceptos;
      $datosFactura['Impuestos']['TotalImpuestosTrasladados'] = $impuesto;
      $datosFactura['Impuestos']['Traslados'][0]['Base'] = $base;
      $datosFactura['Impuestos']['Traslados'][0]['Impuesto'] = '002'; //002 = IVA, 003 = IEPS
      $datosFactura['Impuestos']['Traslados'][0]['TipoFactor'] = 'Tasa'; //Tasa, Cuota, Exento
      $datosFactura['Impuestos']['Traslados'][0]['TasaOCuota'] = $details->tax;
      $datosFactura['Impuestos']['Traslados'][0]['Importe'] = $impuesto;

      // Call api
      $request = $client->post($config->get('api_endpoint').'/api/v5/invoice/create', [
        'headers' => ['X-Api-Key' => $config->get('api_key')],
        'form_params' => [ 'json' => json_encode($datosFactura)]
      ]);

      $response_body = $request->getBody();
      $data  = json_decode($response_body->getContents());
      if($data->code == '200') {
        // Save all transaction data in DB for future reference.
        $this->database->insert('ims_invoices')->fields([
          'sid' => $sales['id'],
          'folio' => $folio['folio'] ? $folio['folio'] + 1 : $config->get('folio'),
          'uuid' => $data->cfdi->UUID,
          'created' => $data->cfdi->FechaTimbrado,
          'pdf' => $data->cfdi->PDF,
          'xml' => $data->cfdi->XML,
          'p_general' => $p_general ? 1 : 0,
        ])->execute();
        if(!$p_general){
          // Send invoice email
          $this->sendInvoice($data->cfdi->UUID, $user['mail']);
        }
        return $data;
      } else {
        return $data->message ?? t('An error has occurred.');
      }
    } catch (RequestException $e) {
      if ($e->hasResponse()) {
        $exception = $e->getResponse()->getBody();
        $exception = json_decode($exception);
        return $exception->message ?? t('An error has occurred.');
      } else {
        \Drupal::logger('IMS')->error($e->getMessage());
        return $e->getMessage();
      }
    }
  }

  /**
   *  Email the customer their invoice
   *
   *  @param string $uuid
   *  Invoice uuid
   *  @param string $email
   *  User email
   *
   */
  function sendInvoice($uuid, $email){
    $client = \Drupal::httpClient();
    $config =  \Drupal::config('ims.settings');
    try {
      // Send the invoice by mail
      $request = $client->post($config->get('api_endpoint').'/api/v5/invoice/send', [
        'headers' => [
          'X-Api-Key' => $config->get('api_key'),
          'uuid' => $uuid,
          'recipient' => $email,
          'bbc' => '',
          'message' => 'Comprobante Fiscal Digital',
        ],
      ]);
      $response_body = $request->getBody();
      $data  = json_decode($response_body->getContents());
      return $data->message;
    } catch (RequestException $e) {
      if ($e->hasResponse()) {
        $exception = $e->getResponse()->getBody();
        $exception = json_decode($exception);
        return $exception->message ?? t('An error occurred while sending the email');
      } else {
        \Drupal::logger('IMS')->error($e->getMessage());
        return t('An error occurred while sending the email.');
      }
    }
  }

  /**
   *  Generate invoices for the general public
   *
   */
  public function globalInvoice() {
    $config =  \Drupal::config('ims.settings');
    // First day of the current month
    $first_day = strtotime(date("Y-m-01"));
    // Start date and time for invoice generation
    $start_day = strtotime(date("Y-m-t ".$config->get('time')."").$config->get('stamp_date'));
    $today = \Drupal::time()->getRequestTime();
    // Last day of the current month
    $last_day = strtotime(date("Y-m-t 23:45:00").$config->get('stamp_date'));
    
    // Validate date and time
    if(($today > $start_day) && ($today < $last_day)) {
      // Get recurring payments that have not been invoiced from the first to the last day of the month
      $query = $this->database->select('ppss_sales_details', 's');
      $query->leftJoin('ims_invoices', 'i', 's.id = i.sid');
      $query->condition('s.created', array($first_day, $last_day), 'BETWEEN');
      $query->fields('s', ['id']);
      $query->fields('i',['uuid']);
      $query->range(0, $config->get('num_rows'));
      $num = 0;
      $results = $query->execute()->fetchAll();
      foreach($results as $result) {
        // If there is no invoice
        if(!$result->uuid) {
          // Create invoice
          $invoice = $this->createInvoice(true, $result->id);
          if($invoice->code ?? false && $invoice->code == '200') {
            // Count invoices
            $num = $num + 1;
          } else {
            \Drupal::logger('IMS')->info(t("Error when creating sales invoice @id - @invoice", 
              ["@id" => $result->id, '@invoice' => $invoice]));
          }
        }
      }
      if($num > 0) {
        \Drupal::logger('IMS')->info(t("@num invoices were generated for the general public", ["@num" => $num]));
      }
    }
  }
}
