<?php

namespace Drupal\ims\Plugin\Block;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;

/**
 * Provides a invoicing block with all needed fields.
 *
 * @Block(
 *   id = "invoice_block",
 *   admin_label = @Translation("IMS Invoicing Block"),
 * )
 */
class IMSInvoiceBlock extends BlockBase {

  /**
  * {@inheritdoc}
  */
  public function build() {
    $note = $this->t("If you do not invoice at the time of purchase, you have until the penultimate day of 
    month to generate it, otherwise, is generated as an invoice for the General Public.");

    if (\Drupal::currentUser()->isAuthenticated()) {
      // Get current user billing data
      $currentUser = \Drupal::database()->select('ims_user_invoice', 'i')
                ->condition('uid', \Drupal::currentUser()->id())
                ->fields('i')
                ->execute()
                ->fetchAssoc();
      if ($currentUser > 0) {
        // Get the user's last purchase
        $saleResult = \Drupal::database()->select('ppss_sales', 's')
                      ->condition('uid', \Drupal::currentUser()->id())
                      ->fields('s', ['id','details'])
                      ->orderBy('created', 'DESC')
                      ->execute()
                      ->fetchAll();
        $url = Url::fromRoute('ims.invoice', ["id" => $saleResult[0]->id]);
        $description = "<br>".$this->t("Click the button to generate your invoice")."<br><br>
        <b>Note: </b>".$note;
      } else {
        $url = Url::fromRoute('ims.user_data_form', ["user" => \Drupal::currentUser()->id()]);
        $description = "<br>".$this->t("Please register your tax information as it appears on your 
        <a href='https://www.sat.gob.mx/aplicacion/53027/genera-tu-constancia-de-situacion-fiscal' 
        target='_blank'>Constancia de Situación Fiscal</a> to generate your invoice.")."<br><br>
        <b>Note:</b>".$note;
      }
      $data['title'] = [
        '#type' => 'markup',
        '#markup' => $this->t("<h3>Billing Information</h3>")
      ];
      $data['url'] = [
        '#type' => 'link',
        '#title' => $this->t($currentUser > 0 ? "Generate invoice" : "Enter billing information"),
        '#url' => $url,
        '#attributes' => [
          'class' => [
            'button',
          ],
        ]
      ];
      $data['description'] = [
        '#type' => 'markup',
        '#markup' => $description
      ];
      return $data;
    } else {
      return [
        '#markup' => "<h5>".$this->t("Billing details")."</h5>
        ".$this->t('To generate your invoice you need to <a href="/user/login">Log in</a> and enter 
        your tax information in billing tab, As it appear in your "Constancia de Situación Fiscal"').
        "<br><br><b>Note:</b>".$note
      ];
    }
  }

  /**
   * @return int
   */
  public function getCacheMaxAge() {
    return 0;
  }

  /**
   * {@inheritdoc}
   */
  public function blockAccess(AccountInterface $account)
  {
    // If viewing a node, get the fully loaded node object.
    $node = \Drupal::routeMatch()->getParameter('node');

    if (!(is_null($node))) {
      $request = \Drupal::request();
      $requestUri = $request->getRequestUri();

      if (strchr($requestUri, \Drupal::config('ppss.settings')->get('success_url'))) {
        return AccessResult::allowedIfHasPermission($account, 'view ims block');
      }
      
    }

    return AccessResult::forbidden();
  }
  
}